@@ -17,7 +17,7 @@
 # along with veh.  If not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import with_statement
-__version__ = "0.87"
+__version__ = "0.90.0"
 
 from cmd import Cmd
 import sys
@@ -114,15 +114,20 @@ def make_venv(repo):
     # could have a "make virtualenv config with a possible 'internal' value"
     venvpath = _new_venv_path(repo)
     _popencmd(["virtualenv", "--no-site-packages", venvpath])
-    with open("%s/.startup_rc" % venvpath, "w") as out:
-        print >>out, "source %s\nsource %s\n" % (
-            expanduser("~/.bashrc"),
-            "%s/bin/activate" % venvpath
-            )
+    write_startup_rc(venvpath)
     _mark_venv_active(repo, venvpath)
     return venvpath
 
 
+def write_startup_rc(venvpath):
+    with open("%s/.startup_rc" % venvpath, "w") as out:
+        if sys.platform != 'darwin' and os.path.exists(expanduser("~/.bashrc")):
+            print >>out , "source %s\n" % expanduser("~/.bashrc")
+        elif sys.platform == 'darwin' and os.path.exists(expanduser("~/.bash_profile")):
+            print >>out , "source %s\n" % expanduser("~/.bash_profile")
+        print >>out , "source %s\n" % ("%s/bin/activate" % venvpath)
+
+
 def _rm_r(dir):
     "Recursively remove directory."
     for r, dirs, files in os.walk(dir, topdown=False):
@@ -136,7 +141,7 @@ def _rm_r(dir):
                 os.rmdir(os.path.join(r, dn))
     os.rmdir(dir)
 
-def _venvsh(root, venvdir, shellcommand=None, pipe=False):
+def _venvsh(root, venvdir, shellcommand=None, exec_=False, pipe=False):
     """Run the shellcommand inside the specified venv.
 
     If the shellcommand is None then a new interactive shell is
@@ -148,7 +153,8 @@ def _venvsh(root, venvdir, shellcommand=None, pipe=False):
     shell, interactive or otherwise.
     """
 
-    command = [os.environ["SHELL"]]
+    sh = os.environ['SHELL']
+    command = [sh]
     env = None
 
     if os.environ.get("VEHACTIVE", None) != root:
@@ -172,7 +178,10 @@ def _venvsh(root, venvdir, shellcommand=None, pipe=False):
     if _verbose:
         print >>sys.stderr,  "running %s inside the venv %s in %s" % (command, venvdir, root)
 
-    return _popencmd(command, env=env, pipe=pipe)
+    if exec_:
+        os.execve(sh, command, env if env is not None else os.environ.copy())
+    else:
+        return _popencmd(command, env=env, pipe=pipe)
 
 
 def cleanup_inactive_venvs(repo):
@@ -563,7 +572,7 @@ will NOT run ipython in the virtualenv.
         # shells do not run an rcfile on startup when given a command.
         root = self._getroot()
         vmdir = venv(root)
-        _venvsh(root, vmdir, " ".join(shellcmd))
+        _venvsh(root, vmdir, " ".join(shellcmd), exec_=True)
 
     def do_noop(self, arg):
         """No-op. Just checks the virtualenv.
@@ -601,11 +610,7 @@ will NOT run ipython in the virtualenv.
             sys.exit(1)
 
         # Now rewrite the startup rc file
-        with open("%s/.startup_rc" % newvenv, "w") as out:
-            print >>out, "source %s\nsource %s\n" % (
-                expanduser("~/.bashrc"),
-                "%s/bin/activate" % newvenv
-                )
+        write_startup_rc(newvenv)
 
         # ... and finally mark it as the active one
         _mark_venv_active(root, newvenv)
